// Copyright 2017 SiegeLord
// Licensed under GPL, see LICENSE for full terms

use allegro::*;
use allegro_font::*;

use engine::id_map::HasId;
use game_state::{Object, GameState};
use errors::Res;
use std::rc::Rc;

pub struct IntroMode
{
	title: Rc<Bitmap>,
	level_filename: String,
}

pub fn create_intro_mode(level_filename: &str, state: &mut GameState) -> Res<Object>
{
	let mut obj = Object::new(state.id_mint.new_id());
	
	obj.intro_mode = Some(IntroMode
	{
		title: state.bitmap_cache.load(&state.core, "data/intro.png")?,
		level_filename: level_filename.into(),
	});
	
	Ok(obj)
}

simple_behavior!
{
	IntroModeDraw[obj.intro_mode.is_some()] |obj, state|
	{
		let obj = obj.borrow();
		let mode = obj.intro_mode.as_ref().unwrap();

		let core = &state.core;
		let ui_font = &state.ui_font;
		
		let bmp = &*mode.title;
		core.draw_bitmap(bmp, 0.0, 0.0, Flag::zero());
		
		let text = 
		["I am a Succubus, a Demon entrusted with harvesting souls for the",
		"Eternal War against the Devils. I have infiltrated a platoon of ",
		"soldiers as a battlefield medic. Unfortunately, the Cunning devils",
		"have launched an attack to rob me of my catch. I cannot fight them ",
		"directly, but I can aid my soldiers.",
		"",
		"Perhaps in the midst of combat I may yet harvest some souls...",
		"",
        "Press SHIFT to kiss. My kisses are deadly to a man, but deliciously ",
        "distracting to a devil.",
        "",
        "Press SPACE to heal. Unfortunately my healing applies to both friend ",
        "and foe...",
        "",
        "Use ARROWS for movement."];
        
		let pink = Color::from_rgb(255, 130, 206);
		
		let hx = 16.0;
		let hy = 16.0;
		
		let spacing = 16.0;
		let mut y = hy + 32.0;
		
		for line in &text
		{
			core.draw_text(ui_font, pink, hx, y, FontAlign::Left, line);
			y += spacing;
		}
	}
}

simple_behavior!
{
	IntroModeInput[obj.intro_mode.is_some()] |obj, state|
	{
		let mut obj = obj.borrow_mut();
		let menu_id = obj.get_id();
		
		let mode = obj.intro_mode.as_mut().unwrap();
		
		if state.key_down.is_some()
		{
			let mode = ::game_mode::create_game_mode(&mode.level_filename, state)?;
			state.add_object(mode);
			state.remove_object(menu_id);
			state.start_fade = true;
		}
	}
}
