// Copyright 2017 SiegeLord
// Licensed under GPL, see LICENSE for full terms

use allegro::*;
use allegro_font::*;

use engine::id_map::HasId;
use game_state::{Object, GameState};
use errors::Res;
use std::rc::Rc;
use game_mode::{load_maps, MapDef};

static NUM_ENTRIES: usize = 3;

pub struct MenuMode
{
	title: Rc<Bitmap>,
	cur_sel: usize,
	map_sel: usize,
	maps: Vec<MapDef>,
}

pub fn create_menu_mode(state: &mut GameState) -> Res<Object>
{
	let mut obj = Object::new(state.id_mint.new_id());
	
	obj.menu_mode = Some(MenuMode
	{
		title: state.bitmap_cache.load(&state.core, "data/title.png")?,
		cur_sel: 0,
		map_sel: 0,
		maps: load_maps("data/levels")?,
	});
	
	state.sfx.play_music("data/alejadeszczu-fixed.xm", &state.audio);
	
	Ok(obj)
}

simple_behavior!
{
	MenuModeDraw[obj.menu_mode.is_some()] |obj, state|
	{
		let obj = obj.borrow();
		let mode = obj.menu_mode.as_ref().unwrap();

		let core = &state.core;
		let ui_font = &state.ui_font;
		
		let bmp = &*mode.title;
		core.draw_bitmap(bmp, 0.0, 0.0, Flag::zero());
		
		let white = Color::from_rgb_f(1.0, 1.0, 1.0);
		let pink = Color::from_rgb(255, 130, 206);
		
		let hx = 500.0;
		let hy = 200.0;
		
		let spacing = 16.0;
		let mut y = hy + 32.0;
		
		let get_sel_color = |cur_sel, entry: usize|
		{
			if entry == cur_sel
			{
				white
			}
			else
			{
				pink
			}
		};
		
		core.draw_text(ui_font, get_sel_color(mode.cur_sel, 0), hx, y, FontAlign::Left, "Start");		
		y += spacing;
		
		core.draw_text(ui_font, get_sel_color(mode.cur_sel, 1), hx, y, FontAlign::Left, "Map");
		y += spacing;
		
		core.draw_text(ui_font, get_sel_color(mode.cur_sel, 1), hx, y, FontAlign::Left, "<          >");
		core.draw_text(ui_font, get_sel_color(mode.cur_sel, 1), hx + 52.0, y, FontAlign::Centre, &format!("{}", &mode.maps[mode.map_sel].level.name));
		y += spacing;
		
		core.draw_text(ui_font, get_sel_color(mode.cur_sel, 2), hx, y, FontAlign::Left, "Quit");
	}
}

simple_behavior!
{
	MenuModeInput[obj.menu_mode.is_some()] |obj, state|
	{
		let mut obj = obj.borrow_mut();
		let menu_id = obj.get_id();
		
		let mut mode = obj.menu_mode.as_mut().unwrap();
		
		let mut start = false;
		
		if let Some(k) = state.key_down
		{
			match k
			{
				KeyCode::Up =>
				{
					mode.cur_sel = if mode.cur_sel == 0
					{
						NUM_ENTRIES - 1
					}
					else
					{
						mode.cur_sel - 1
					};
					state.play_sound("data/ui_sound1.wav")?;
				}
				KeyCode::Down =>
				{
					mode.cur_sel = if mode.cur_sel == NUM_ENTRIES - 1
					{
						0
					}
					else
					{
						mode.cur_sel + 1
					};
					state.play_sound("data/ui_sound1.wav")?;
				}
				KeyCode::Left =>
				{
					if mode.cur_sel == 1
					{
						mode.map_sel = if mode.map_sel == 0
						{
							mode.maps.len() - 1
						}
						else
						{
							mode.map_sel - 1
						};
						state.play_sound("data/ui_sound1.wav")?;
					}
				}
				KeyCode::Right =>
				{
					if mode.cur_sel == 1
					{
						mode.map_sel = if mode.map_sel == mode.maps.len() - 1
						{
							0
						}
						else
						{
							mode.map_sel + 1
						};
						state.play_sound("data/ui_sound1.wav")?;
					}
				}
				KeyCode::Space | KeyCode::Enter =>
				{
					match mode.cur_sel
					{
						0 => 
						{
							start = true;
							state.play_sound("data/ui_sound2.wav")?;
						}
						2 => 
						{
							state.quit = true;
						}
						_ => ()
					}
				}
				KeyCode::Escape =>
				{
					state.quit = true;
				}
				_ => ()
			}
		}
		
		if start
		{
			let mode = ::intro_mode::create_intro_mode(&mode.maps[mode.map_sel].level_filename, state)?;
			state.add_object(mode);
			state.remove_object(menu_id);
			state.start_fade = true;
		}
	}
}
